"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var helpers_1 = require("./helpers");
function createSwitch() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    return function (arg) {
        var defaultCase = undefined;
        var tests = args.slice();
        while (tests.length > 0) {
            var result = tests.pop()(arg);
            if (result.default) {
                defaultCase = result;
                continue;
            }
            if (result.success) {
                return result.value;
            }
        }
        if (defaultCase !== undefined) {
            return defaultCase.value;
        }
    };
}
exports.createSwitch = createSwitch;
function createSwitchDefault(value) {
    return function (arg) {
        return {
            default: true,
            success: false,
            value: value,
        };
    };
}
exports.createSwitchDefault = createSwitchDefault;
function createSwitchOption(match, value) {
    return function (arg) {
        if (arg === match) {
            return { default: false, success: true, value: value };
        }
        return { default: false, success: false, value: undefined };
    };
}
exports.createSwitchOption = createSwitchOption;
function fold(left, right) {
    return function (arg) { return (arg ? right : left); };
}
exports.fold = fold;
function ifElse(condition, left, right) {
    var resolveIfIsPipe = function (context, currentPipe, value) {
        return helpers_1.isContextFlowAsync(context) ? currentPipe.resolve(value) : currentPipe.resolveSync(value);
    };
    return function (value, context) {
        helpers_1.throwIfNotFunction("ifElse", 'condition', condition);
        var result = context.call(condition);
        if (helpers_1.isPipe(left) && !result) {
            return resolveIfIsPipe(context, left, value);
        }
        else if (helpers_1.isPipe(right) && result) {
            return resolveIfIsPipe(context, right, value);
        }
        else if (result) {
            return right;
        }
        else {
            return left;
        }
    };
}
exports.ifElse = ifElse;
