"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var helpers_1 = require("./helpers");
function add(value) {
    return function (arg) {
        helpers_1.throwIfNotNumber('add', arg);
        return arg + value;
    };
}
exports.add = add;
function changeSign() {
    return function (arg) {
        helpers_1.throwIfNotNumber('changeSign', arg);
        return -arg;
    };
}
exports.changeSign = changeSign;
function decrement() {
    return function (arg) {
        helpers_1.throwIfNotNumber('decrement', arg);
        return --arg;
    };
}
exports.decrement = decrement;
function divideBy(value) {
    return function (arg) {
        helpers_1.throwIfNotNumber('divideBy', arg);
        return arg / value;
    };
}
exports.divideBy = divideBy;
function increment() {
    return function (arg) {
        helpers_1.throwIfNotNumber('increment', arg);
        return ++arg;
    };
}
exports.increment = increment;
function multiplyBy(value) {
    return function (arg) {
        helpers_1.throwIfNotNumber('multiplyBy', arg);
        return arg * value;
    };
}
exports.multiplyBy = multiplyBy;
function negative() {
    return function (arg) {
        helpers_1.throwIfNotNumber('negative', arg);
        return arg > 0 ? -arg : arg;
    };
}
exports.negative = negative;
function positive() {
    return function (arg) {
        helpers_1.throwIfNotNumber('positive', arg);
        return arg > 0 ? arg : -arg;
    };
}
exports.positive = positive;
function pow(exponent) {
    return function (arg) {
        helpers_1.throwIfNotNumber('pow', arg);
        return Math.pow(arg, exponent);
    };
}
exports.pow = pow;
function root(exponent) {
    return function (arg) {
        helpers_1.throwIfNotNumber('root', arg);
        return Math.pow(arg, 1 / exponent);
    };
}
exports.root = root;
function subtract(value) {
    return function (arg) {
        helpers_1.throwIfNotNumber('subtract', arg);
        return arg - value;
    };
}
exports.subtract = subtract;
function takeBetween(start, end) {
    return function (arg) {
        helpers_1.throwIfNotArray("takeBetween", arg);
        var newArray = [];
        var length = arg.length;
        for (var i = 0; i < length; i++) {
            var value = arg[i];
            helpers_1.throwIfNotNumber('takeBetween', value);
            if (value >= start && value <= end) {
                newArray.push(value);
            }
        }
        return newArray;
    };
}
exports.takeBetween = takeBetween;
function takeGreater() {
    return function (arg) {
        helpers_1.throwIfNotArray('takeGreater', arg);
        return Math.max.apply(Math, arg);
    };
}
exports.takeGreater = takeGreater;
function takeGreaterThan(check, equal) {
    if (equal === void 0) { equal = false; }
    return function (arg) {
        helpers_1.throwIfNotArray("takeGreaterThan", arg);
        var newArray = [];
        var length = arg.length;
        for (var i = 0; i < length; i++) {
            var value = arg[i];
            helpers_1.throwIfNotNumber('takeGreaterThan', value);
            if (equal && value >= check) {
                newArray.push(value);
            }
            else if (value > check) {
                newArray.push(value);
            }
        }
        return newArray;
    };
}
exports.takeGreaterThan = takeGreaterThan;
function takeLower() {
    return function (arg) {
        helpers_1.throwIfNotArray('takeLower', arg);
        return Math.min.apply(Math, arg);
    };
}
exports.takeLower = takeLower;
function takeLowerThan(check, equal) {
    if (equal === void 0) { equal = false; }
    return function (arg) {
        helpers_1.throwIfNotArray("takeLowerThan", arg);
        var newArray = [];
        var length = arg.length;
        for (var i = 0; i < length; i++) {
            var value = arg[i];
            helpers_1.throwIfNotNumber('takeLowerThan', value);
            if (equal && value <= check) {
                newArray.push(value);
            }
            else if (value < check) {
                newArray.push(value);
            }
        }
        return newArray;
    };
}
exports.takeLowerThan = takeLowerThan;
function takeOuter(start, end) {
    return function (arg) {
        helpers_1.throwIfNotArray("takeOuter", arg);
        var newArray = [];
        var length = arg.length;
        for (var i = 0; i < length; i++) {
            var value = arg[i];
            helpers_1.throwIfNotNumber('takeOuter', value);
            if (value < start || value > end) {
                newArray.push(value);
            }
        }
        return newArray;
    };
}
exports.takeOuter = takeOuter;
