"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var helpers_1 = require("./helpers");
function apply(pipe) {
    return function (arg, context) {
        helpers_1.throwContextExecutionFlow('apply', context, 'async');
        return pipe.resolve(arg);
    };
}
exports.apply = apply;
function applySync(pipe) {
    return function (arg, context) {
        helpers_1.throwContextExecutionFlow('applySync', context, 'sync');
        return pipe.resolveSync(arg);
    };
}
exports.applySync = applySync;
/**
 * Puts a value inside a pipeline
 * @export
 * @template TValue
 * @param {TValue} value
 * @returns {() => TValue}
 */
function put(value) {
    return function () { return value; };
}
exports.put = put;
/**
 * @export
 * @template T
 * @template TValue
 * @template TReturn
 * @param {T} callback
 * @returns {((arg: TValue) => TReturn | void)}
 */
function safeCall(callback, fallback) {
    return function (arg) {
        try {
            return callback(arg);
        }
        catch (_a) {
            return fallback;
        }
    };
}
exports.safeCall = safeCall;
function useCallValue() {
    return function (_, context) { return context.callValue; };
}
exports.useCallValue = useCallValue;
function useValue() {
    return function (value) { return value; };
}
exports.useValue = useValue;
