"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var helpers_1 = require("./helpers");
var util_1 = require("util");
function exclude() {
    var keys = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        keys[_i] = arguments[_i];
    }
    return function (arg) {
        helpers_1.throwIfNotObject("pick", arg);
        var newObject = __assign({}, arg);
        var keysArray = keys.slice();
        while (keysArray.length > 0) {
            delete newObject[keysArray.pop()];
        }
        return newObject;
    };
}
exports.exclude = exclude;
function hasProperty(propertyKey) {
    return function (arg) {
        if (util_1.isNullOrUndefined(arg)) {
            return false;
        }
        return Object.prototype.hasOwnProperty.call(arg, propertyKey);
    };
}
exports.hasProperty = hasProperty;
function keys() {
    return function (arg) {
        helpers_1.throwIfNotObject("keys", arg);
        return Object.keys(arg);
    };
}
exports.keys = keys;
function maybe(path, fallbackValue) {
    var shouldUseFallback = function (current) {
        return util_1.isNullOrUndefined(current) && !util_1.isNullOrUndefined(fallbackValue);
    };
    var returnFallbackValue = function (context) {
        if (!helpers_1.isPipe(fallbackValue)) {
            return fallbackValue;
        }
        var isAsync = context.executionFlow === 'async';
        if (isAsync) {
            return fallbackValue.resolve(undefined);
        }
        return fallbackValue.resolveSync(undefined);
    };
    return function (arg, context) {
        helpers_1.throwIfNotObject('maybe', arg);
        var pathChunks = path.split('.').reverse();
        var current = arg;
        while (pathChunks.length) {
            var key = pathChunks.pop();
            if (!current) {
                break;
            }
            current = current[key];
        }
        if (shouldUseFallback(current)) {
            return returnFallbackValue(context);
        }
        return current;
    };
}
exports.maybe = maybe;
function merge(target) {
    return function (arg) {
        helpers_1.throwIfNotObject('merge', arg);
        return __assign({}, arg, target);
    };
}
exports.merge = merge;
function pick() {
    var keys = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        keys[_i] = arguments[_i];
    }
    return function (arg) {
        helpers_1.throwIfNotObject("pick", arg);
        var newObject = {};
        var keysArray = keys.slice();
        while (keysArray.length > 0) {
            var key = keysArray.pop();
            if (arg[key]) {
                newObject[key] = arg[key];
            }
        }
        return newObject;
    };
}
exports.pick = pick;
