"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var helpers_1 = require("./helpers");
var regexps = {};
function camelCase() {
    return function (arg) {
        helpers_1.throwIfNotString("camelCase", arg);
        return arg
            .replace(/(?:^\w|[A-Z]|\b\w)/g, function (letter, index) { return (index === 0 ? letter.toLowerCase() : letter.toUpperCase()); })
            .replace(/\s+/g, '');
    };
}
exports.camelCase = camelCase;
function chars() {
    return function (arg) {
        helpers_1.throwIfNotString("chars", arg);
        return arg.split('');
    };
}
exports.chars = chars;
function concat(str) {
    return function (arg) {
        helpers_1.throwIfNotString("concat", arg);
        return arg + str;
    };
}
exports.concat = concat;
function includes(str) {
    return function (arg) {
        helpers_1.throwIfNotString('find', arg);
        return arg.indexOf(str) >= 0;
    };
}
exports.includes = includes;
function length() {
    return function (arg) {
        helpers_1.throwIfNotString("length", arg);
        return arg.length;
    };
}
exports.length = length;
function lowercase() {
    return function (arg) {
        helpers_1.throwIfNotString("lowercase", arg);
        return arg.toLowerCase();
    };
}
exports.lowercase = lowercase;
function pascalCase() {
    return function (arg, context) {
        var camelized = camelCase()(arg, context);
        return camelized.charAt(0).toUpperCase() + camelized.slice(1);
    };
}
exports.pascalCase = pascalCase;
function repeat(count) {
    if (count === void 0) { count = 1; }
    return function (arg) {
        helpers_1.throwIfNotString('repeat', arg);
        var i = 0;
        var newArg = arg;
        while (i < count) {
            newArg += arg;
            i++;
        }
        return newArg;
    };
}
exports.repeat = repeat;
function replaceAll(needle, replaceWith) {
    return function (arg) {
        helpers_1.throwIfNotString("replaceAll", arg);
        regexps[needle] = regexps[needle] || new RegExp(needle, 'gi');
        return arg.replace(regexps[needle], replaceWith);
    };
}
exports.replaceAll = replaceAll;
function toBinaryArray() {
    var binarizeCharset = function (index, charset) {
        if (index >= charset.length) {
            return charset;
        }
        var binary = charset[index].charCodeAt(0).toString(2);
        var newCharset = charset.slice();
        newCharset.splice(index, 1, binary);
        return binarizeCharset(index + 1, newCharset);
    };
    return function (arg, context) {
        helpers_1.throwIfNotString("toBinary", arg);
        return binarizeCharset(0, context.call(chars()));
    };
}
exports.toBinaryArray = toBinaryArray;
function uppercase() {
    return function (arg) {
        helpers_1.throwIfNotString("uppercase", arg);
        return arg.toUpperCase();
    };
}
exports.uppercase = uppercase;
